package cn.com.qmth.stmms.ms.marking.api;

import cn.com.qmth.stmms.ms.commons.web.PageableDTO;
import cn.com.qmth.stmms.ms.core.domain.ExamQuestion;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTask;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.repository.ExamQuestionRepo;
import cn.com.qmth.stmms.ms.core.repository.LevelRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskRepo;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import cn.com.qmth.stmms.ms.marking.assembler.MarkTaskAssembler;
import cn.com.qmth.stmms.ms.marking.dto.MarkTaskDTO;
import cn.com.qmth.stmms.ms.marking.service.MarkingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.*;

/**
 * 评卷任务数据api
 * Created by zhengmin on 2016/10/12.
 */
@RestController
@RequestMapping("api/marktasks")
public class MarkTaskApi {

    @Autowired
    private MarkTaskRepo markTaskRepo;

    @Autowired
    private MarkingService markingService;

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private MarkTaskAssembler markTaskAssembler;

    @Autowired
    ExamQuestionRepo examQuestionRepo;

    @Autowired
    PaperRepo paperRepo;

    /**
     * 评卷员的评卷任务
     *
     * @param markerId   评卷员用户id
     * @param stage      评卷阶段
     * @param level      档位
     * @param sn         密号
     * @param questionId 试题id
     * @return
     */
    @RequestMapping(method = RequestMethod.GET)
    public PageableDTO list(@RequestParam Long markerId,
                            @RequestParam MarkStage stage,
                            @RequestParam(required = false) Boolean isSample,
                            @RequestParam(required = false) String level,
                            @RequestParam(required = false) String sn,
                            @RequestParam Long questionId,
                            Pageable pageable) throws Exception {
        List<MarkTaskDTO> markTaskDTOs = new ArrayList<>();

        Specification<MarkTask> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(builder.equal(root.get("questionId"), questionId));
            predicates.add(builder.equal(root.get("markerId"), markerId));
            predicates.add(builder.equal(root.get("stage"), stage));
            if (level == null) {
                predicates.add(builder.isNull(root.get("result")));
            } else if (stage == MarkStage.LEVEL) {
                predicates.add(builder.equal(root.get("result"), level));
            } else if (stage == MarkStage.SCORE) {
                predicates.add(builder.isNotNull(root.get("result")));
            }
            if (isSample != null) {
                predicates.add(builder.equal(root.get("paper").get("isSample"), isSample));
            }
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Page<MarkTask> markTasks = markTaskRepo.findAll(specification, pageable);
        if (Objects.isNull(markTasks) || markTasks.getContent().size() == 0) {
            if (Objects.nonNull(isSample) && isSample) {
                ExamQuestion examQuestion = examQuestionRepo.findOne(questionId);
                Paper paper = paperRepo.findTopByLevelAndIsSampleAndIsMissingAndSubjectAndTestOrderByUpdatedOnDesc(level, isSample, false, examQuestion.getSubject(), TrialEnum.START_TRIAL.getId());
                ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
                HttpServletRequest request = attributes.getRequest();
                HttpSession session = request.getSession();
                String token = (String) session.getAttribute("token");
                if (Objects.isNull(token)) {
                    throw new Exception("请先登录");
                }
                MarkUser markUser = (MarkUser) session.getAttribute("markUser");
                MarkTask markTask = new MarkTask(markUser, paper, stage, 1L);
                markTaskDTOs.add(markTaskAssembler.toDTO(markTask));
            }
        }
        markTasks.getContent().forEach(m -> {
            markTaskDTOs.add(markTaskAssembler.toDTO(m));
        });
        if (stage == MarkStage.SCORE && level == null) {
            Collections.sort(markTaskDTOs, (MarkTaskDTO a, MarkTaskDTO b) -> a.getLevel().compareTo(b.getLevel()));
        }
        return new PageableDTO(markTaskDTOs, markTasks.getTotalElements(), markTasks.getTotalPages(), pageable.getPageNumber());
    }

    /**
     * 提交评卷任务
     *
     * @param markTask 评卷你任务id
     * @param body     评卷内容
     * @return
     */
    @RequestMapping(value = "{markTask}", method = RequestMethod.PATCH)
    public ResponseEntity marking(@PathVariable MarkTask markTask, @RequestBody HashMap<String, String> body) {
        MarkStage stage = MarkStage.valueOf(body.get("stage"));
        String result = body.get("result");
        boolean oldRejected = markTask.isRejected();
        if (stage == null || result == null) {
            return new ResponseEntity(HttpStatus.BAD_REQUEST);
        }
        switch (stage) {
            case LEVEL:
                markingService.levelMark(markTask, result);
                break;
            case SCORE:
                Integer score = Integer.parseInt(result);
                markingService.scoring(markTask, score);
                break;
        }
        return new ResponseEntity(markTaskAssembler.toDTO(markTask, oldRejected), HttpStatus.OK);
    }

    /**
     * 跳过评卷任务
     *
     * @param markTaskId 评卷你任务id
     * @return
     */
    @RequestMapping(value = "{markTaskId}/skip", method = RequestMethod.POST)
    public ResponseEntity skip(@PathVariable Long markTaskId) {
        MarkTask markTask = markingService.skip(markTaskId);
        return new ResponseEntity(markTaskAssembler.toDTO(markTask), HttpStatus.OK);
    }

}
